/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.area.views;

import net.wordrider.area.RiderStyles;
import net.wordrider.utilities.Swinger;
import net.wordrider.utilities.Utils;

import javax.swing.text.*;
import java.awt.*;

/**
 * @author Vity
 */
final class RiderLabelView extends LabelView {
    private boolean overline = false;
    private boolean dotted = false;
    private boolean _underline = false;
    private boolean wordWrap = false;
    private boolean math = false;
    private boolean inverse = false;
    private boolean nextElementVector = false;
    //private boolean isBigSize = false;

    public RiderLabelView(final Element elem) {
        super(elem);
    }

    public final Color getForeground() {
        return (math) ? RiderStyles.getMathStyleColor() : super.getForeground();
    }

    public final View breakView(final int axis, final int p0, final float pos, final float len) {
        if (wordWrap)
            return super.breakView(axis, p0, pos, len);
        //wordwrap is on
        if (axis == View.X_AXIS) {
            final int p1 = this.getGlyphPainter().getBoundedPosition(this,
                    p0, pos, len);            
            final RiderLabelView v = (RiderLabelView) createFragment(p0, p1);
            return v;
        }
        return this;
    }


    public final Font getFont() {
        final Font f = super.getFont();    //call to super
        return (!f.getFamily().equals(RiderStyles.FONT_FAMILY)) ? RiderStyles.getAreaFont() : f;
    }

    protected final void setPropertiesFromAttributes() {
        super.setPropertiesFromAttributes();
        final AttributeSet attr = getAttributes();
        if (attr != null) {
            wordWrap = RiderStyles.isWordWrap(attr);
            math = RiderStyles.isMath(getElement());
            //bold = StyleConstants.isBold(attr);
            // doubleline = (attr.getAttribute("2UNDERLINE") != null);
            dotted = (RiderStyles.isUnderLineDotted(attr));
            overline = RiderStyles.isVector(attr);
            inverse = RiderStyles.isInvert(attr);
            if (overline) {
                nextElementVector = isNextElementVector(getElement(), getDocument());
            }
            //isBigSize = RiderStyles.isSize(attr, RiderStyles.SIZE_MAXI);
            _underline = RiderStyles.isUnderLine(attr);
            //overline = net.wordrider.area.RiderStyles.isVector(attr);
            //dotted = (net.wordrider.area.RiderStyles.isUnderLineDotted(attr));
        }
    }

    private final static boolean isNextElementVector(final Element el, final Document doc) {
        final Element parent = el.getParentElement();
        final int elCount = parent.getElementCount();
        boolean foundElement = false;
        //ElementIterator aa = new ElementIterator();
        Element itEl;
        for (int i = 0; i < elCount; ++i) {
            itEl = parent.getElement(i);
            if (foundElement) {
                if (RiderStyles.isVector(itEl.getAttributes())) {
                    if (i + 1 == elCount) {
                        try {
                            return !doc.getText(itEl.getStartOffset(), itEl.getEndOffset() - itEl.getStartOffset()).equals("\n");
                        } catch (BadLocationException e) {
                            Utils.processException(e);
                            return false;
                        }
                    } else
                        return true;
                } else
                    return false;
            } else if (itEl.equals(el))
                foundElement = true;
        }
        return false;
    }

    private void fixDrawBug(final Graphics g, final int x1, final int yStart, final int yHigh) {
        g.setPaintMode();
        final JTextComponent component = (JTextComponent) getContainer();
        if (RiderViewFactory.isInSelection(component, getElement()))
            g.setColor(component.getSelectionColor());
        else
            g.setColor(component.getBackground());
        g.drawLine(x1, yStart + 1, x1, yHigh + 1);
    }

    public final void paint(final Graphics g, final Shape a) {
        // g.setFont(font);
        if (Swinger.ANTIALIASING) {
            Graphics2D g2d = (Graphics2D) g;
            g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
                    RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
            g2d.setRenderingHint(RenderingHints.KEY_RENDERING,
                    RenderingHints.VALUE_RENDER_QUALITY);
        }
        super.paint(g, a);
        //Color bg = getBackground();

//            if (bg != null) {
//                g.setColor(bg);
//                g.fillRect(alloc.x, alloc.y, alloc.width, alloc.height);
//            }

//            final boolean strike = isStrikeThrough();
        if (_underline || overline || dotted || inverse) {
            final int p0 = getStartOffset();
            final int p1 = getEndOffset();
            final Rectangle alloc = (a instanceof Rectangle) ? (Rectangle) a : a.getBounds();
            g.setColor(getForeground());
            checkPainter();
            final GlyphPainter painter = getGlyphPainter();
            //painter.paint(this, g, a, p0, p1);
            //if (underline || strike || overline || dotted) {
//
//                    View parent = getParent();
//                    if ((parent != null) && (parent.getEndOffset() == p1)) {	// strip whitespace on end
//                        Segment s = getText(p0, p1);
//                        while ((s.count > 0) && (Character.isSpace(s.array[s.count - 1]))) {
//                            p1 -= 1;
//                            s.count -= 1;
//                        }
//                    }

            int x0 = alloc.x;
            final int p = getStartOffset();
            if (p != p0) x0 += (int) painter.getSpan(this, p, p0, getTabExpander(), x0);
            final int x1 = x0 + (int) painter.getSpan(this, p0, p1, getTabExpander(), x0);
            // calculate y coordinate
            final int d = (int) painter.getDescent(this);
            int yTmp = alloc.y + alloc.height - d;

            if (inverse && x0 == x1)
            //repaint fix bug
                fixDrawBug(g, x0, yTmp + 1, alloc.y + alloc.height - (int) painter.getAscent(this) - 4);

//            if (bold) {
//                final Graphics2D g2 = (Graphics2D) g;
//                g2.setStroke(new BasicStroke(2.0f));
//            }

            Stroke stroke = null;
            //if (_underline || dotted) {
            if (dotted) {
                final Graphics2D g2 = (Graphics2D) g;
                stroke = g2.getStroke();
                g2.setStroke(RiderStyles.DOTTED_STROKE);
            }
            if (dotted) {
                ++yTmp;
                //  if (bold) yTmp++;
                g.drawLine(x0, yTmp, x1, yTmp);
//                    if (doubleline) {
//                        yTmp += 2;
//                        if (bold) yTmp++;
//                        g.drawLine(x0, yTmp, x1, yTmp);
//                    }
            } else if (_underline && (x1 == x0))
                fixDrawBug(g, x0, yTmp, yTmp);

            if (stroke != null) {
                final Graphics2D g2 = (Graphics2D) g;
                g2.setStroke(stroke);
            }
//                if (strike) {
//                    yTmp -= (int) (painter.getAscent(this) * 0.3f);
//                    g.drawLine(x0, yTmp, x1, yTmp);
//                }
            if (overline) {
                yTmp = alloc.y + alloc.height - (int) painter.getAscent(this) - 1;
                if (x0 != x1)
                    g.drawLine(x0, yTmp, x1, yTmp);
                if (x1 != x0 && !nextElementVector) {
                    g.drawLine(x1 + 1, yTmp, x1 - 4, yTmp - 2);
                    g.drawLine(x1 + 1, yTmp, x1 - 4, yTmp + 2);
                }
//                    if (doubleover) {
//                        yTmp -= 2;
//                        if (bold) yTmp--;
//                        g.drawLine(x0, yTmp, x1, yTmp);
//                    }
            }

        }
    }
}