/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.area.views;

import net.wordrider.area.RiderStyles;
import net.wordrider.utilities.Swinger;

import javax.swing.*;
import javax.swing.text.Element;
import javax.swing.text.JTextComponent;
import javax.swing.text.StyleConstants;
import javax.swing.text.View;
import java.awt.*;

final class RiderParagraphView extends javax.swing.text.ParagraphView {

    int childCount = 0;
    int shift = 0;
    private static JTextComponent editor;
    private static int rowPosition = -1;
    private static int newPosition;
    private View childView;
    private static Image breakPointImage = null;
    private static Image mathPointImage = null;

    public RiderParagraphView(final Element e) {
        super(e);
        if (breakPointImage == null) {
            ImageIcon imageIcon = Swinger.getIcon("breakpoint.gif");
            if (imageIcon != null)
                breakPointImage = imageIcon.getImage();
        }
        if (mathPointImage == null) {
            ImageIcon imageIcon = Swinger.getIcon("math.gif");
            if (imageIcon != null)
                mathPointImage = imageIcon.getImage();
        }

    }

    protected void setPropertiesFromAttributes() {
        super.setPropertiesFromAttributes();    //call to super        
        if (RiderStyles.isImage(getElement().getElement(0)) != null) {
            setJustification(StyleConstants.ALIGN_CENTER);
        }
    }
    
    public final void paint(final Graphics g, final Shape a) {
        childCount = this.getViewCount();
        super.paint(g, a);
        //rowCountInThisParagraph = this.getViewCount(); //<----- YOU HAVE REAL ROW COUNT FOR ONE PARAGRAPH}

//            Rectangle r = g.getClipBounds();
//            g.setColor(new Color(0xC0C0C0));
//            g.drawLine(r.x, r.y, r.x + r.width, r.y + r.height);
    }

    public final void paintChild(final Graphics g, final Rectangle r, final int n) {
        super.paintChild(g, r, n);
        if (n == 0) {
            editor = (JTextComponent) getContainer();
            if (RiderStyles.isBookmark(getElement())) {
                if (breakPointImage != null) {
                    g.drawImage(breakPointImage, 7, r.y + r.height / 2 - 7, null);
                } else {
                    g.setColor(Color.RED);
                    g.fillOval(6, r.y + r.height / 2 - 5, 10, 10);
                }
            }else
            if (RiderStyles.isMath(getElement())) {
                if (mathPointImage != null) {
                    g.drawImage(mathPointImage, 4, r.y + r.height / 2 - 8, null);
                } else {
                    g.setColor(Color.BLUE);
                    g.fillOval(6, r.y + r.height / 2 - 5, 10, 10);
                }
            }
            else {
                g.setColor(Color.RED);
                g.fillOval(10, r.y + (r.height / 2) - 2, 3, 3);
            }
        }
        newPosition = shift + n + 1;
        if (newPosition != rowPosition) {
            final int curPosition = editor.getCaretPosition();
            childView = getView(n);
            if (childView.getStartOffset() <= curPosition && curPosition <= childView.getEndOffset())
                editor.firePropertyChange("linePosition", rowPosition, rowPosition = newPosition);
            rowPosition = newPosition;
        }
    }


}