/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.core.actions;

import net.wordrider.core.managers.AreaManager;
import net.wordrider.core.managers.FileInstance;
import net.wordrider.dialogs.CloseDialog;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.util.Collection;
import java.util.Iterator;

/**
 * @author Vity
 */
public final class CloseAllAction extends CoreAction {
    private static final CloseAllAction instance = new CloseAllAction();
    private static final String CODE = "CloseAllAction";

    private CloseAllAction() {
        super(CODE, KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK), null);
    }


    public static final CloseAllAction getInstance() {
        return instance;
    }

    public final boolean closeAll(final ActionEvent e) {
        final AreaManager areaManager = getAreaManager();
        FileInstance fi;
        final Collection modifiedList = areaManager.getModifiedInstances();
        if (modifiedList.size() > 0) {
            final CloseDialog dialog = new CloseDialog(getMainFrame(), modifiedList);
            final Collection selectedList = dialog.getReturnList();
            if (selectedList != null) {
                //closes all except the selected on the list
                for (final Iterator it = areaManager.getOpenedInstances().iterator(); it.hasNext();) {
                    if (!selectedList.contains(fi = (FileInstance) it.next())) {
                        areaManager.closeInstanceHard(fi);
                    }
                }
                for (final Iterator it = selectedList.iterator(); it.hasNext();) {
                    fi = (FileInstance) it.next();
                    areaManager.setActivateFileInstance(fi);
                    if (SaveFileAction.getInstance().save(e, false) == false)
                        return false;
                    areaManager.closeActiveInstance();
                }
            } else
                return false;//user set cancel
        } else {
            for (final Iterator it = areaManager.getOpenedInstances().iterator(); it.hasNext();) {
                areaManager.closeInstanceHard((FileInstance) it.next());
                //it.remove();
            }
        }
        return true;
    }

    public final void actionPerformed(final ActionEvent e) {
        closeAll(e);
    }
}
