/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.core.managers;

import net.wordrider.area.RiderArea;
import net.wordrider.core.AppPrefs;
import net.wordrider.core.MainApp;
import net.wordrider.core.actions.OpenFileAction;
import net.wordrider.core.actions.SaveFileAction;
import net.wordrider.core.actions.CloseActiveAction;
import net.wordrider.core.managers.interfaces.IFileInstance;
import net.wordrider.core.managers.interfaces.IInformedTab;
import net.wordrider.ti89.TITextFileInfo;
import net.wordrider.utilities.Swinger;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.JTextComponent;
import java.awt.*;
import java.io.File;

/**
 * @author Vity
 */
public final class FileInstance implements IFileInstance, IInformedTab, Comparable {

    private File file = null;
    private static int untitledCount;
    private String name;
    private final RiderArea area = new RiderArea();
    private final JScrollPane scrollPane = new JScrollPane();

    private TITextFileInfo fileInfo;
//    private Integer instanceID;
    private ManagerDirector director;


    public FileInstance() {
        setFileInfo(new TITextFileInfo());
        init();
    }

    public FileInstance(final File file, final TITextFileInfo fileInfo) {
        setFileInfo(fileInfo);
        this.file = file;
        init();
    }

    private void init() {
        scrollPane.getViewport().add(area);
        scrollPane.setFocusable(false);
        scrollPane.setFocusCycleRoot(false);
        scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        updateName();
        this.director = MainApp.getInstance().getMainAppFrame().getManagerDirector();
    }

    private void updateName() {
        this.name = (this.file == null) ? AppPrefs.getLabel("editor.untitled", String.valueOf(++untitledCount)) : this.file.getName();
    }

    public int compareTo(Object o) {
        return getName().compareTo(o.toString());
    }

    public final File getFile() {
        return this.file;
    }

    public final String getName() {
        return name;
    }

    public final TITextFileInfo getFileInfo() {
        return fileInfo;
    }

    public final void setFileInfo(final TITextFileInfo fileInfo) {
        this.fileInfo = fileInfo;
    }

    public final void setFile(final File file) {
        this.file = file;
        updateName();
        director.getAreaManager().setTabTitle(this); //save as
        CloseActiveAction.getInstance().updateStatusName(this);
        director.getStatusbarManager().displayFilePath(file.getAbsolutePath());
        ((RiderArea) area).setModified(false);
    }

    public final JTextComponent getTextComponent() {
        return this.area;
    }

    public final Component getComponent() {
        return scrollPane;
    }

    public FileFilter getFileFilter() {
        return new OpenFileAction.File89tFilter();
    }

    public final Icon getIcon() {
        return null;
    }

    public final String getTip() {
        return (file != null) ? file.getAbsolutePath() : null;
    }

    public final void activate() {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                area.requestFocus();
            }
        });
    }

    public String getTabName() {
        return (isModified() ? getName() + "*" : getName());
    }

    public boolean closeSoft() {
        if (area.isModified())
            switch (Swinger.getChoiceCancel(MainApp.getInstance().getMainAppFrame(), AppPrefs.getLabel("message.confirm.changed"))) {
                case Swinger.RESULT_YES:
                    SaveFileAction.getInstance().actionPerformed(null);
                    break;
                case Swinger.RESULT_NO:
                    closeHard();
                    return true;
                default:
                    return false;
            }
        closeHard();
        return true;
    }

    public final String toString() {
        return getName();
    }

    public void closeHard() {
        area.finalize();
    }
//    public final void setInstanceID(final Integer instanceID) {
//        this.instanceID = instanceID;
//    }
    public boolean isModified() {
        return area.isModified();
    }

    public final void deactivate() {
        //implement - call to super class
    }
}
