/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.core.managers;

import net.wordrider.area.RiderArea;
import net.wordrider.area.RiderStyles;
import net.wordrider.area.actions.*;
import net.wordrider.core.AppPrefs;
import net.wordrider.core.MainApp;
import net.wordrider.core.actions.CreateNewFileAction;
import net.wordrider.core.actions.OpenFileAction;
import net.wordrider.core.actions.SaveAsFileAction;
import net.wordrider.core.actions.SaveFileAction;
import net.wordrider.core.managers.interfaces.IAreaChangeListener;
import net.wordrider.core.managers.interfaces.IFileInstance;
import net.wordrider.core.managers.interfaces.IHideAble;
import net.wordrider.core.managers.interfaces.IRiderManager;
import net.wordrider.gui.ToolbarSeparator;
import net.wordrider.utilities.Utils;

import javax.swing.*;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.swing.text.MutableAttributeSet;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

/**
 * @author Vity
 */
public class ToolbarManager implements IRiderManager, CaretListener, IAreaChangeListener, DocumentListener, PropertyChangeListener, IHideAble {
    private JPanel panel;
    private JToolBar toolbar;
    private JToggleButton leftItem, centerItem, rightItem, margin10Item, margin20Item, margin30Item, mathItem;
    private JToggleButton bookmarkItem;
  //  private JButton cutItem, copyItem, pasteItem, searchItem, separatorSingleItem, separatorDoubleItem, pictureItem;
    private JToggleButton size1Item, size2Item, size3Item, superScriptItem, underlineSingleItem, underlineDottedItem, strikeoutItem, invertItem, vectorItem, wordWrapItem;
    //private JButton newItem, openItem, saveItem, saveasItem, undoItem, redoItem;

    private final static Dimension buttonDimension = new Dimension(24, 23);
    private static final int STRUT_SIZE = 8;
    private RiderArea editor;
    private boolean activeMath = false;

    public ToolbarManager() {
        panel = new JPanel(new BorderLayout());
        panel.setBorder(BorderFactory.createEmptyBorder(0, 2, 0, 0));
        UIManager.addPropertyChangeListener(this);

        createToolbar();
        panel.setVisible(AppPrefs.getProperty("net.wordrider.gui.showToolbar", true));
    }

    private void createToolbar() {
        panel.add(toolbar = new JToolBar("Main Toolbar"));

        panel.setPreferredSize(new Dimension(400, 27));
        SwingUtilities.updateComponentTreeUI(toolbar);
        //toolbar.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 0));
        //toolbar.setLayout(null);
//        toolbar.setLayout(new BoxLayout(toolbar, BoxLayout.X_AXIS));
//        toolbar.setLayout(new GridBagLayout());
        toolbar.setFocusable(false);
        toolbar.setFloatable(false);

        final UpdateActionListener updateListener = new UpdateActionListener();

        toolbar.add(getButton(CreateNewFileAction.getInstance()));
        toolbar.add(getButton(OpenFileAction.getInstance()));
        toolbar.add(Box.createHorizontalStrut(2));
        toolbar.add(getButton(SaveFileAction.getInstance()));
        toolbar.add(getButton(SaveAsFileAction.getInstance()));
        toolbar.add(new ToolbarSeparator());
        toolbar.add(getButton(UndoAction.getInstance()));
        toolbar.add(getButton(RedoAction.getInstance()));
        toolbar.add(new ToolbarSeparator());
        toolbar.add(getButton(ShowFindReplaceDialogAction.getInstance()));
        toolbar.add(new ToolbarSeparator());
        toolbar.add(getButton(CutAction.getInstance()));
        toolbar.add(getButton(CopyAction.getInstance()));
        toolbar.add(getButton(PasteAction.getInstance()));
        toolbar.add(new ToolbarSeparator());
        ButtonGroup group = new ButtonGroup();
        toolbar.add(size1Item = getToggleButton(ChangeFontSizeStyle1Action.getInstance()));
        group.add(size1Item);
        toolbar.add(size2Item = getToggleButton(ChangeFontSizeStyle2Action.getInstance()));
        group.add(size2Item);
        toolbar.add(size3Item = getToggleButton(ChangeFontSizeStyle3Action.getInstance()));

        group.add(size3Item);
        toolbar.add(Box.createHorizontalStrut(STRUT_SIZE));
        toolbar.add(superScriptItem = getToggleButton(ChangeExponentStyleAction.getInstance()));
        toolbar.add(new ToolbarSeparator());

        toolbar.add(underlineSingleItem = getToggleButton(ChangeUnderlineStyleAction.getInstance()));
        underlineSingleItem.addActionListener(updateListener);
        toolbar.add(underlineDottedItem = getToggleButton(ChangeDottedStyleAction.getInstance()));
        toolbar.add(Box.createHorizontalStrut(STRUT_SIZE));

        toolbar.add(strikeoutItem = getToggleButton(ChangeStrikedStyleAction.getInstance()));

        toolbar.add(new ToolbarSeparator());

        toolbar.add(invertItem = getToggleButton(ChangeInvertStyleAction.getInstance()));
        toolbar.add(vectorItem = getToggleButton(ChangeVectorStyleAction.getInstance()));
        toolbar.add(wordWrapItem = getToggleButton(ChangeWordWrapStyleAction.getInstance()));

        toolbar.add(new ToolbarSeparator());

        group = new ButtonGroup();

        group.add(leftItem = getToggleButton(AligmentLeftAction.getInstance()));
        toolbar.add(leftItem);

        group.add(centerItem = getToggleButton(AligmentCenterAction.getInstance()));
        toolbar.add(centerItem);

        toolbar.add(rightItem = getToggleButton(AligmentRightAction.getInstance()));
        group.add(rightItem);
        toolbar.add(Box.createHorizontalStrut(STRUT_SIZE));

        toolbar.add(margin10Item = getToggleButton(Margin10Action.getInstance()));
        group.add(margin10Item);

        toolbar.add(margin20Item = getToggleButton(Margin20Action.getInstance()));
        group.add(margin20Item);

        toolbar.add(margin30Item = getToggleButton(Margin30Action.getInstance()));
        group.add(margin30Item);
        toolbar.add(Box.createHorizontalStrut(STRUT_SIZE));
        toolbar.add(mathItem = getToggleButton(ChangeMathStyleAction.getInstance()));
        toolbar.add(new ToolbarSeparator());

        toolbar.add(getButton(InsertPictureAction.getInstance()));
        toolbar.add(Box.createHorizontalStrut(STRUT_SIZE));
        toolbar.add(getButton(InsertSeparatorSingleAction.getInstance()));
        toolbar.add(getButton(InsertSeparatorDoubleAction.getInstance()));
        toolbar.add(Box.createHorizontalStrut(STRUT_SIZE));
        toolbar.add(bookmarkItem = getToggleButton(UpdateBreakpointAction.getInstance()));
        panel.validate();
    }

    private static final JToggleButton getToggleButton(final Action action) {
        final JToggleButton button = new JToggleButton(action);
        //button.setToolTipText(tooltip);
        button.setAction(action);
        button.setText(null);
        button.setMinimumSize(buttonDimension);
        button.setPreferredSize(buttonDimension);
        button.setMaximumSize(buttonDimension);
        button.setMnemonic(0);
        button.setFocusable(false);
        return button;
    }

    private class UpdateActionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            updateStatus();            
        }
    }

    public void insertUpdate(DocumentEvent e) {
        updateStatus();
    }

    private void unregisterHooks() {
        editor.getDocument().removeDocumentListener(this);
        editor.removePropertyChangeListener("document", this);
        editor.removeCaretListener(this);
        editor.removePropertyChangeListener(StatusbarManager.MODIFIED_PROPERTY, this);
    }

    public final void selectedAreaChanged(final IFileInstance instance) {
        if (instance != null && isVisible()) {
            if (!instance.getTextComponent().equals(editor)) {
                if (editor != null)
                    unregisterHooks();
                registerHooks((RiderArea) instance.getTextComponent());
            }
            updateModifiedStatus();
            updateButtonsStatuses();
        } else if (editor != null) {
            unregisterHooks();
            editor = null;
        }
    }


    private void updateButtonsStatuses() {
        if (editor == null)
            return;
        final MutableAttributeSet set = editor.getInputAttributes();

        final Element paraEl = editor.getDoc().getParagraphElement(editor.getCaretPosition());
        final boolean isNOTMath = !RiderStyles.isMath(paraEl);

        if (activeMath != isNOTMath) {
            ChangeFontSizeStyle1Action.getInstance().setEnabled(isNOTMath);
            ChangeFontSizeStyle2Action.getInstance().setEnabled(isNOTMath);
            ChangeFontSizeStyle3Action.getInstance().setEnabled(isNOTMath);
            AligmentLeftAction.getInstance().setEnabled(isNOTMath);
            AligmentCenterAction.getInstance().setEnabled(isNOTMath);
            AligmentRightAction.getInstance().setEnabled(isNOTMath);
            Margin10Action.getInstance().setEnabled(isNOTMath);
            Margin20Action.getInstance().setEnabled(isNOTMath);
            Margin30Action.getInstance().setEnabled(isNOTMath);
            ChangeInvertStyleAction.getInstance().setEnabled(isNOTMath);
            ChangeVectorStyleAction.getInstance().setEnabled(isNOTMath);
            ChangeWordWrapStyleAction.getInstance().setEnabled(isNOTMath);
            ChangeExponentStyleAction.getInstance().setEnabled(isNOTMath);
            ChangeUnderlineStyleAction.getInstance().setEnabled(isNOTMath);
            ChangeDottedStyleAction.getInstance().setEnabled(isNOTMath);
            ChangeStrikedStyleAction.getInstance().setEnabled(isNOTMath);
            UpdateBreakpointAction.getInstance().setEnabled(isNOTMath);
            activeMath = isNOTMath;
            mathItem.setSelected(!isNOTMath);
        }

        size1Item.setSelected(RiderStyles.isSize(set, RiderStyles.SIZE_MINI));
        size2Item.setSelected(RiderStyles.isSize(set, RiderStyles.SIZE_NORMAL));
        size3Item.setSelected(RiderStyles.isSize(set, RiderStyles.SIZE_MAXI));
        superScriptItem.setSelected(RiderStyles.isExposant(set));

        if (RiderStyles.isUnderLineDotted(set)) {
            underlineSingleItem.setSelected(false);
            underlineDottedItem.setSelected(true);
        } else if (RiderStyles.isUnderLine(set)) {
            underlineSingleItem.setSelected(true);
            underlineDottedItem.setSelected(false);
        } else {
            underlineSingleItem.setSelected(false);
            underlineDottedItem.setSelected(false);
        }

        switch (RiderStyles.getMargin(set)) {
            case RiderStyles.MARGIN_10:
                margin10Item.setSelected(true);
                break;
            case RiderStyles.MARGIN_20:
                margin20Item.setSelected(true);
                break;
            case RiderStyles.MARGIN_30:
                margin30Item.setSelected(true);
                break;
            case RiderStyles.MARGIN_0:
            default:
                switch (RiderStyles.getAligment(set)) {
                    case RiderStyles.ALIGN_LEFT:
                        leftItem.setSelected(true);
                        break;
                    case RiderStyles.ALIGN_CENTER:
                        centerItem.setSelected(true);
                        break;
                    case RiderStyles.ALIGN_RIGHT:
                        rightItem.setSelected(true);
                        break;
                    default:
                        leftItem.setSelected(true);
                        break;
                }
                break;
        }
        invertItem.setSelected(RiderStyles.isInvert(set));
        vectorItem.setSelected(RiderStyles.isVector(set));
        wordWrapItem.setSelected(RiderStyles.isWordWrap(set));
        strikeoutItem.setSelected(RiderStyles.isStrikeOut(set));
        bookmarkItem.setSelected(RiderStyles.isBookmark(paraEl));

        final boolean isSelectedText = editor.getSelectionStart() != editor.getSelectionEnd();
        CopyAction.getInstance().setEnabled(isSelectedText && editor.isEditable());
        CutAction.getInstance().setEnabled(isSelectedText && editor.isEditable());
        PasteAction.getInstance().setEnabled(editor.isEditable());
    }

    public final Component getManagerComponent() {
        return panel;  //implement - call to super class
    }

    private void updateModifiedStatus() {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                SaveFileAction.getInstance().setEnabled((editor != null && editor.isModified()));
            }
        });
    }

    public final void updateStatus() {
        if (Utils.isDefinedForLog(Utils.DEBUG_LOG))
            Utils.log(Utils.DEBUG_LOG, "update toolbar buttons statuses");
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                updateButtonsStatuses();
            }
        });
    }

    public final void caretUpdate(CaretEvent e) {
        updateStatus();
    }

    public final void changedUpdate(DocumentEvent e) {
        updateStatus();
    }


    public final void removeUpdate(DocumentEvent e) {
        updateStatus();
    }

    public final void propertyChange(PropertyChangeEvent event) {
        if (event.getSource().equals(editor) &&
                event.getPropertyName().equals("document")) {
            // Reset the DocumentListener
            ((Document) event.getOldValue()).removeDocumentListener(this);
            final Document newDoc = (Document) event.getNewValue();
            newDoc.addDocumentListener(this);
        } else if (event.getPropertyName().equals(StatusbarManager.MODIFIED_PROPERTY)) {
            updateModifiedStatus();
        } else if (event.getPropertyName().equals("lookAndFeel")) {
            panel.remove(toolbar);
            createToolbar();
            updateButtonsStatuses();
//            SwingUtilities.updateComponentTreeUI(toolbar);
            toolbar.invalidate();
            toolbar.validate();
            toolbar.repaint();
        }
    }

    private void registerHooks(final RiderArea area) {
        editor = area;
        editor.addCaretListener(this);
        editor.getDocument().addDocumentListener(this);
        editor.addPropertyChangeListener("document", this);
        editor.addPropertyChangeListener(StatusbarManager.MODIFIED_PROPERTY, this);
    }

    private static final JButton getButton(final Action action) {
        final JButton button = new JButton(action);
        button.setText("");
        button.setMnemonic(0);
        button.setMinimumSize(buttonDimension);
        button.setPreferredSize(buttonDimension);
        button.setMaximumSize(buttonDimension);
        button.setFocusable(false);
        return button;
    }

    public final boolean isVisible() {
        return panel.isVisible();
    }

    public final void setVisible(final boolean value) {
        if (value) {
            panel.setVisible(true);
            selectedAreaChanged(MainApp.getInstance().getMainAppFrame().getManagerDirector().getAreaManager().getActiveFileInstance());
        } else {
            panel.setVisible(false);
            selectedAreaChanged(null);
        }
        AppPrefs.storeProperty("net.wordrider.gui.showToolbar", value);
    }


}
