/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.ti89;

import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

/**
 * @author Vity
 */
public abstract class TIFileWriter extends TIFile {

    public final boolean saveToFile(File file) throws IOException {
        DataOutputStream stream = null;
        try {
            if (!file.exists() && !file.createNewFile()) return false;
            stream = new DataOutputStream(new FileOutputStream(file));
            writeHeader(stream);
            writeContent(stream);
            writeChecksum(stream);
            return true;
        } finally {
            if (stream != null) {
                stream.close();
            }
        }
    }

    private static void writeString(DataOutputStream stream, String text, int length) throws IOException {
        final byte[] arrayOfBytes = new String(text.getBytes(), "ASCII").getBytes();
        stream.write(arrayOfBytes, 0, (arrayOfBytes.length > length) ? length : arrayOfBytes.length);
        //fill by zeroes
        if ((arrayOfBytes.length < length))
            for (int i = arrayOfBytes.length; i < length; ++i) stream.writeByte(0x0);
    }


    private void writeHeader(DataOutputStream stream) throws IOException {
        stream.writeBytes(getFileHeaderType());
        stream.writeShort(0x1 << 8);
        writeString(stream, fileInfo.getFolderName(), 8);
        writeString(stream, fileInfo.getComment(), 40);
        stream.writeShort(0x1 << 8);//number of vars = 1
        stream.writeInt(0x52 << 24); //data address
        writeString(stream, fileInfo.getVarName(), 8);
        stream.writeShort(getDataType() << 8); //data type - text        
        stream.writeShort(0xFF << 8); //data type - text
    }

    private void writeChecksum(DataOutputStream stream) throws IOException {
        final int crcsum = getCheckSum();
        stream.writeByte(crcsum);// hi a low bytes of crc sum
        stream.writeByte(crcsum >> 8);
    }


    protected abstract void writeContent(DataOutputStream stream) throws IOException;

}