/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.ti89;

import java.io.DataInputStream;
import java.io.IOException;

/**
 * @author Vity
 */
public final class TITextFileReader extends TIFileReader {
    private final static String[] supportedFormats = {"**TI92P*", "**TI89**"};
    private String content = null;

    public TITextFileReader() {
        this.fileInfo = new TITextFileInfo();
    }

    public final TITextFileInfo getTextFileInfo() {
        return (TITextFileInfo) fileInfo;
    }

    protected final void readContent(DataInputStream stream) throws IOException {
        stream.skipBytes(4);
        int character;
        final StringBuffer buffer = new StringBuffer();
        while (true) {
            character = stream.readUnsignedByte();
            if (character == 0 || character == -1)
                break;
            else
                buffer.append((char) character);
        }
        if (character == 0 || character == -1)
            stream.skipBytes(1);//0xE0
        content = buffer.toString();
    }


    protected final boolean isSupportedHeader(String header) {
        final int size = supportedFormats.length;
        for (int i = 0; i < size; ++i)
            if (supportedFormats[i].equals(header))
                return true;
        return false;  //implement - call to super class
    }

    public final String getContent() {
        return content;
    }

    public final int getDataType() {
        return DATA_TYPES[DATA_TYPE_TEXT];  //implement - call to super class
    }

    protected final String getFileHeaderType() {
        return supportedFormats[0];  //implement - call to super class
    }


    protected final char getCheckSum() {
        if (content == null) return 0;
        final int length = content.length();
        final int length4 = content.length() + 0x4;
        final char[] array = content.toCharArray();
        char result = 0;
        for (int i = 0; i < length; ++i)
            result += array[i];
        return (char) (result + 0xE0 + 0x1 + ((length4 & 0xff)+((length4 >> 8) & 0xff)));  //+0x20
    }
}
