/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.ti89;

/**
 * 
 * @author Vity 
 */

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.HashMap;

public final class TITextFileWriter extends TIFileWriter {

    private static final String fileHeader = "**TI92P*"; //readable headers
    private String textContent = "";
    private static final HashMap UNICODE2TI;
    private static int sum = 0;


    static {
        UNICODE2TI = new HashMap(134);
        UNICODE2TI.put(new Character('\u00D0'), new Character((char) 18));
        UNICODE2TI.put(new Character('\u00BB'), new Character((char) 22));
        UNICODE2TI.put(new Character('\u00BC'), new Character((char) 28));
        UNICODE2TI.put(new Character('\u00BD'), new Character((char) 29));
        UNICODE2TI.put(new Character('\u00BE'), new Character((char) 30));
        UNICODE2TI.put(new Character('\u00BF'), new Character((char) 31));
        UNICODE2TI.put(new Character('\u20AC'), new Character((char) 128));
        UNICODE2TI.put(new Character('\u0081'), new Character((char) 129));
        UNICODE2TI.put(new Character('\u201A'), new Character((char) 130));
        UNICODE2TI.put(new Character('\u0192'), new Character((char) 131));
        UNICODE2TI.put(new Character('\u201E'), new Character((char) 132));
        UNICODE2TI.put(new Character('\u2026'), new Character((char) 133));
        UNICODE2TI.put(new Character('\u2020'), new Character((char) 134));
        UNICODE2TI.put(new Character('\u2021'), new Character((char) 135));
        UNICODE2TI.put(new Character('\u02C6'), new Character((char) 136));
        UNICODE2TI.put(new Character('\u2030'), new Character((char) 137));
        UNICODE2TI.put(new Character('\u0160'), new Character((char) 138));
        UNICODE2TI.put(new Character('\u2039'), new Character((char) 139));
        UNICODE2TI.put(new Character('\u0152'), new Character((char) 140));
        UNICODE2TI.put(new Character('\u008D'), new Character((char) 141));
        UNICODE2TI.put(new Character('\u017D'), new Character((char) 142));
        UNICODE2TI.put(new Character('\u008F'), new Character((char) 143));
        UNICODE2TI.put(new Character('\u0090'), new Character((char) 144));
        UNICODE2TI.put(new Character('\u2018'), new Character((char) 145));
        UNICODE2TI.put(new Character('\u2019'), new Character((char) 146));
        UNICODE2TI.put(new Character('\u0081'), new Character((char) 129));
        UNICODE2TI.put(new Character('\u201D'), new Character((char) 148));
        UNICODE2TI.put(new Character('\u2013'), new Character((char) 149));
        UNICODE2TI.put(new Character('\u2014'), new Character((char) 150));
        UNICODE2TI.put(new Character('\u00C6'), new Character((char) 151));
        UNICODE2TI.put(new Character('\u02DC'), new Character((char) 152));
        UNICODE2TI.put(new Character('\u2122'), new Character((char) 153));
        UNICODE2TI.put(new Character('\u0161'), new Character((char) 154));
        UNICODE2TI.put(new Character('\u203A'), new Character((char) 155));
        UNICODE2TI.put(new Character('\u0153'), new Character((char) 156));
        UNICODE2TI.put(new Character('\u009D'), new Character((char) 157));
        UNICODE2TI.put(new Character('\u017E'), new Character((char) 158));
        UNICODE2TI.put(new Character('\u0178'), new Character((char) 159));
        UNICODE2TI.put(new Character('\u00A1'), new Character((char) 160));
        UNICODE2TI.put(new Character('\u00A2'), new Character((char) 161));
        UNICODE2TI.put(new Character('\u00A3'), new Character((char) 162));
        UNICODE2TI.put(new Character('\u00A4'), new Character((char) 163));
        UNICODE2TI.put(new Character('\u00A5'), new Character((char) 164));
        UNICODE2TI.put(new Character('\u00A6'), new Character((char) 165));
        UNICODE2TI.put(new Character('\u007C'), new Character((char) 166));
        UNICODE2TI.put(new Character('\u00A7'), new Character((char) 167));
        UNICODE2TI.put(new Character('\u00A7'), new Character((char) 168));
        UNICODE2TI.put(new Character('\u00A8'), new Character((char) 169));
        UNICODE2TI.put(new Character('\u00AA'), new Character((char) 170));
        UNICODE2TI.put(new Character('\u00D7'), new Character((char) 171));
        UNICODE2TI.put(new Character('\u00A9'), new Character((char) 172));
        UNICODE2TI.put(new Character('\u00AA'), new Character((char) 173));
        UNICODE2TI.put(new Character('\u00A8'), new Character((char) 174));
        UNICODE2TI.put(new Character('\u00AB'), new Character((char) 175));
        UNICODE2TI.put(new Character('\u00AC'), new Character((char) 176));
        UNICODE2TI.put(new Character('\u00AD'), new Character((char) 177));
        UNICODE2TI.put(new Character('\u00AE'), new Character((char) 178));
        UNICODE2TI.put(new Character('\u00AF'), new Character((char) 179));
        UNICODE2TI.put(new Character('\u00B0'), new Character((char) 180));
        UNICODE2TI.put(new Character('\u00B1'), new Character((char) 181));
        UNICODE2TI.put(new Character('\u00B6'), new Character((char) 182));
        UNICODE2TI.put(new Character('\u00B2'), new Character((char) 183));
        UNICODE2TI.put(new Character('\u00B3'), new Character((char) 184));
        UNICODE2TI.put(new Character('\u00B4'), new Character((char) 185));
        UNICODE2TI.put(new Character('\u00B5'), new Character((char) 186));
        UNICODE2TI.put(new Character('\u00DE'), new Character((char) 187));
        UNICODE2TI.put(new Character('\u00B6'), new Character((char) 188));
        UNICODE2TI.put(new Character('\u00B7'), new Character((char) 189));
        UNICODE2TI.put(new Character('\u00B8'), new Character((char) 190));
        UNICODE2TI.put(new Character('\u00B9'), new Character((char) 191));
        UNICODE2TI.put(new Character('\u00C0'), new Character((char) 192));
        UNICODE2TI.put(new Character('\u00C1'), new Character((char) 193));
        UNICODE2TI.put(new Character('\u00C2'), new Character((char) 194));
        UNICODE2TI.put(new Character('\u00C3'), new Character((char) 195));
        UNICODE2TI.put(new Character('\u00C4'), new Character((char) 196));
        UNICODE2TI.put(new Character('\u00C5'), new Character((char) 197));
        UNICODE2TI.put(new Character('\u00E6'), new Character((char) 198));
        UNICODE2TI.put(new Character('\u00C7'), new Character((char) 199));
        UNICODE2TI.put(new Character('\u00C8'), new Character((char) 200));
        UNICODE2TI.put(new Character('\u00C9'), new Character((char) 201));
        UNICODE2TI.put(new Character('\u00CA'), new Character((char) 202));
        UNICODE2TI.put(new Character('\u00CB'), new Character((char) 203));
        UNICODE2TI.put(new Character('\u00CD'), new Character((char) 204));
        UNICODE2TI.put(new Character('\u00CC'), new Character((char) 205));
        UNICODE2TI.put(new Character('\u00CD'), new Character((char) 206));
        UNICODE2TI.put(new Character('\u00CF'), new Character((char) 207));
        UNICODE2TI.put(new Character('\u00D0'), new Character((char) 208));
        UNICODE2TI.put(new Character('\u00D1'), new Character((char) 209));
        UNICODE2TI.put(new Character('\u00D2'), new Character((char) 210));
        UNICODE2TI.put(new Character('\u00D3'), new Character((char) 211));
        UNICODE2TI.put(new Character('\u00D4'), new Character((char) 212));
        UNICODE2TI.put(new Character('\u00D5'), new Character((char) 213));
        UNICODE2TI.put(new Character('\u00D6'), new Character((char) 214));
        UNICODE2TI.put(new Character('\u0078'), new Character((char) 215));
        UNICODE2TI.put(new Character('\u00D8'), new Character((char) 216));
        UNICODE2TI.put(new Character('\u00D9'), new Character((char) 217));
        UNICODE2TI.put(new Character('\u00DA'), new Character((char) 218));
        UNICODE2TI.put(new Character('\u00DB'), new Character((char) 219));
        UNICODE2TI.put(new Character('\u00DC'), new Character((char) 220));
        UNICODE2TI.put(new Character('\u00DD'), new Character((char) 221));
        UNICODE2TI.put(new Character('\u00FE'), new Character((char) 222));
        UNICODE2TI.put(new Character('\u00DF'), new Character((char) 223));
        UNICODE2TI.put(new Character('\u00E0'), new Character((char) 224));
        UNICODE2TI.put(new Character('\u00E1'), new Character((char) 225));
        UNICODE2TI.put(new Character('\u00E2'), new Character((char) 226));
        UNICODE2TI.put(new Character('\u00E3'), new Character((char) 227));
        UNICODE2TI.put(new Character('\u00E4'), new Character((char) 228));
        UNICODE2TI.put(new Character('\u00E5'), new Character((char) 229));
        UNICODE2TI.put(new Character('\u00E6'), new Character((char) 230));
        UNICODE2TI.put(new Character('\u00E7'), new Character((char) 231));
        UNICODE2TI.put(new Character('\u00E8'), new Character((char) 232));
        UNICODE2TI.put(new Character('\u00E9'), new Character((char) 233));
        UNICODE2TI.put(new Character('\u00EA'), new Character((char) 234));
        UNICODE2TI.put(new Character('\u00EB'), new Character((char) 235));
        UNICODE2TI.put(new Character('\u00EC'), new Character((char) 236));
        UNICODE2TI.put(new Character('\u00ED'), new Character((char) 237));
        UNICODE2TI.put(new Character('\u00EE'), new Character((char) 238));
        UNICODE2TI.put(new Character('\u00EF'), new Character((char) 239));
        UNICODE2TI.put(new Character('\u00F0'), new Character((char) 240));
        UNICODE2TI.put(new Character('\u00F1'), new Character((char) 241));
        UNICODE2TI.put(new Character('\u00F2'), new Character((char) 242));
        UNICODE2TI.put(new Character('\u00F3'), new Character((char) 243));
        UNICODE2TI.put(new Character('\u00F4'), new Character((char) 244));
        UNICODE2TI.put(new Character('\u00F5'), new Character((char) 245));
        UNICODE2TI.put(new Character('\u00F6'), new Character((char) 246));
        UNICODE2TI.put(new Character('\u00F7'), new Character((char) 247));
        UNICODE2TI.put(new Character('\u00F8'), new Character((char) 248));
        UNICODE2TI.put(new Character('\u00F9'), new Character((char) 249));
        UNICODE2TI.put(new Character('\u00FA'), new Character((char) 250));
        UNICODE2TI.put(new Character('\u00FB'), new Character((char) 251));
        UNICODE2TI.put(new Character('\u00FC'), new Character((char) 252));
        UNICODE2TI.put(new Character('\u00FD'), new Character((char) 253));
        UNICODE2TI.put(new Character('\u00FE'), new Character((char) 254));
        UNICODE2TI.put(new Character('\u00FF'), new Character((char) 255));
    }


    public TITextFileWriter() {
    }


    public final String getTextContent() {
        return textContent;
    }

    public final void setTextContent(String textContent) {
        this.textContent = (textContent == null) ? "" : textContent;
    }

    protected final char getCheckSum() {
        return (char) (sum + 0xE0 + 0x1);
    }

    protected final int getDataType() {
        return DATA_TYPES[DATA_TYPE_TEXT];
    }

    protected final String getFileHeaderType() {
        return fileHeader;
    }

    protected final void writeContent(DataOutputStream stream) throws IOException {
        final char[] content = textContent.toCharArray();
        final int length = content.length;
        final int length4 = length + 0x4;
        final int filesize = stream.size() + 0x12 + length;
        //stream.writeShort(filesize);//LL
        stream.writeByte(filesize & 0xFF);
        stream.writeByte((filesize >> 8) & 0xFF);
        stream.writeByte((filesize >> 24) & 0xFF);
        stream.writeByte((filesize >> 16)& 0xFF);
        stream.writeShort(0xA55A); //2 bytes separator
        stream.writeShort(0x0);// 2 bytes zeroes
        stream.writeInt(length4);
        stream.writeByte(0x0);
        stream.writeByte(0x1);         //start of text
        //stream.writeByte(0x20);
        //writing content
        char ch;
        Character tiChar;
        sum = (length4 & 0xff)+((length4 >> 8) & 0xff);
        for (int i = 0; i < length; ++i) {
            ch = content[i];
            if (ch >= 128) {
                tiChar = (Character) UNICODE2TI.get(new Character(ch));
                if (tiChar != null)
                    ch = tiChar.charValue();
              //  else ch = 0xFF;
            } else if (ch == '\n') ch = '\r';
            sum += ch;
            stream.writeByte(ch);
        }
        stream.writeShort(0xE0);//ende
    }

}
