/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.core.managers;

import net.wordrider.area.RiderArea;
import net.wordrider.core.managers.interfaces.IAreaChangeListener;
import net.wordrider.core.managers.interfaces.IFileChangeListener;
import net.wordrider.core.managers.interfaces.IFileInstance;
import net.wordrider.core.managers.interfaces.IInformedTab;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;

/**
 * @author Vity
 */
public final class AreaManager extends TabManager {
    private final ManagerDirector director;
    private final Collection areaChangelisteners = new ArrayList(4);
    private final Collection fileStatusListeners = new ArrayList(1);
    private final RecentFilesManager recentFilesManager;

    public AreaManager(final ManagerDirector director) {
        super();    //call to super
        this.director = director;
        fileStatusListeners.add(recentFilesManager = new RecentFilesManager(director.getMenuManager()));
    }

    final public IFileInstance getActiveFileInstance() {
        return (IFileInstance) super.getActiveInstance();
    }

    final public IFileInstance isFileAlreadyOpened(final File f) {
        IFileInstance instance;
        for (final Iterator it = runningInstancesIDs.values().iterator(); it.hasNext();) {
            instance = (IFileInstance) it.next();
            if (f.equals(instance.getFile()))
                return instance;
        }
        return null;
    }

    final public void openFileInstance() {
        openFileInstance(new FileInstance());
        //  activateInstance(id, instance);
        // MainApp.getMainApp().getMainAppFrame().setTitle();
        //tabbedPane.validate();
//        tabbedPane.repaint();
    }

    final public void openFileInstance(final FileInstance fileInstance) {
        registerNewOne(fileInstance);
        for (final Iterator it = fileStatusListeners.iterator(); it.hasNext();)
            ((IFileChangeListener) it.next()).fileWasOpened(fileInstance.getFile());
//        fileInstance.getTextComponent().grabFocus();
//        SwingUtilities.invokeLater(
//                new Runnable() {
//                    public void run() {
//                        fileInstance.getTextComponent().grabFocus();
//                    }
//                }
//        );
        //  activateInstance(id, instance);
        // MainApp.getMainApp().getMainAppFrame().setTitle();
        //tabbedPane.validate();
//        tabbedPane.repaint();
    }

    final protected void deactivateInstance(final Integer anID) {
        super.deactivateInstance(anID);    //call to super
        director.getPluginToolsManager().selectedAreaChanged(null);
        IAreaChangeListener listener;
        for (final Iterator it = areaChangelisteners.iterator(); it.hasNext();) {
            listener = (IAreaChangeListener) it.next();
            listener.selectedAreaChanged(null);
        }
    }

    final protected void activateInstance(final Integer anID) {
        super.activateInstance(anID);    //call to super
        final IFileInstance instance = getActiveFileInstance();
        IAreaChangeListener listener;
        for (final Iterator it = areaChangelisteners.iterator(); it.hasNext();) {
            listener = (IAreaChangeListener) it.next();
            listener.selectedAreaChanged(instance);
        }
    }

    public final void setTabTitle(final IInformedTab informedPlugin) {
        final int index = tabbedPane.indexOfComponent(informedPlugin.getComponent());
        tabbedPane.setTitleAt(index, informedPlugin.getTabName());
    }

    final void addAreaChangeListener(final IAreaChangeListener listener) {
        areaChangelisteners.add(listener);
    }

    final void removeAreaChangeListener(final IAreaChangeListener listener) {
        areaChangelisteners.remove(listener);
    }

    public final Collection getOpenedInstances() {
        return new LinkedList(runningInstancesIDs.values());
    }

    public final Collection getModifiedInstances() {
        IFileInstance fi;
        final Collection list = new ArrayList();
        for (final Iterator it = getOpenedInstances().iterator(); it.hasNext();) {
            fi = (IFileInstance) it.next();
            if (((RiderArea) fi.getTextComponent()).isModified())
                list.add(fi);
        }
        return list;
    }


    public final void setActivateFileInstance(final FileInstance instance) {
        tabbedPane.setSelectedComponent(instance.getComponent());
    }

    public final void closeInstanceHard(final FileInstance instance) {
        final File fileInstance = instance.getFile();
        closeHard((Integer) runningTabs.get(instance.getComponent()));
        for (final Iterator it = fileStatusListeners.iterator(); it.hasNext();)
            ((IFileChangeListener) it.next()).fileWasClosed(fileInstance);
    }


    public final void closeActiveInstance() {
        IFileInstance fileInstance = getActiveFileInstance();
        if (fileInstance != null) {
            closeSoft(activeInstanceID);
            final File savedFileInstance = fileInstance.getFile();
            for (final Iterator it = fileStatusListeners.iterator(); it.hasNext();)
                ((IFileChangeListener) it.next()).fileWasClosed(savedFileInstance);
        }
    }

    public RecentFilesManager getRecentFilesManager() {
        return recentFilesManager;
    }

    public void grabActiveFocus() {
        IFileInstance instance = getActiveFileInstance();
        if (instance != null)
            instance.getTextComponent().grabFocus();
    }

    public void updateTabTitle() {        
        setTabTitle((IInformedTab)getActiveInstance());
    }
}
